@@ -208,14 +208,17 @@ gdk_window_init (void)
 		&x, &y, &width, &height, &border_width, &depth);
   XGetWindowAttributes (gdk_display, gdk_root_window, &xattributes);
 
-  gdk_root_parent.xdisplay = gdk_display;
   gdk_root_parent.xwindow = gdk_root_window;
+  gdk_root_parent.xdisplay = gdk_display;
   gdk_root_parent.window_type = GDK_WINDOW_ROOT;
   gdk_root_parent.window.user_data = NULL;
   gdk_root_parent.width = width;
   gdk_root_parent.height = height;
   gdk_root_parent.children = NULL;
   gdk_root_parent.colormap = NULL;
+  gdk_root_parent.ref_count = 1;
+
+  gdk_xid_table_insert (&gdk_root_window, &gdk_root_parent);
 }
 
 GdkWindow*
@@ -703,11 +706,15 @@ gdk_window_unref (GdkWindow *window)
   if (private->ref_count == 0)
     {
       if (!private->destroyed)
-	g_warning ("losing last reference to undestroyed window\n");
+	{
+	  if (private->window_type == GDK_WINDOW_FOREIGN)
+	    gdk_xid_table_remove (private->xwindow);
+	  else
+	    g_warning ("losing last reference to undestroyed window\n");
+	}
       g_dataset_destroy (window);
       g_free (window);
     }
-
 }
 
 void
@@ -1322,6 +1329,62 @@ gdk_window_get_origin (GdkWindow *window,
   return return_val;
 }
 
+void
+gdk_window_get_root_origin (GdkWindow *window,
+			    gint      *x,
+			    gint      *y)
+{
+  GdkWindowPrivate *private;
+  Window xwindow;
+  Window xparent;
+  Window root;
+  Window *children;
+  unsigned int nchildren;
+
+  g_return_if_fail (window != NULL);
+
+  private = (GdkWindowPrivate*) window;
+  if (x)
+    *x = 0;
+  if (y)
+    *y = 0;
+  if (private->destroyed)
+    return;
+      
+  while (private->parent && ((GdkWindowPrivate*) private->parent)->parent)
+    private = (GdkWindowPrivate*) private->parent;
+  if (private->destroyed)
+    return;
+
+  xparent = private->xwindow;
+  do
+    {
+      xwindow = xparent;
+      if (!XQueryTree (private->xdisplay, xwindow,
+		       &root, &xparent,
+		       &children, &nchildren))
+	return;
+
+      if (children)
+	XFree (children);
+    }
+  while (xparent != root);
+
+  if (xparent == root)
+    {
+      unsigned int ww, wh, wb, wd;
+      int wx, wy;
+
+      if (XGetGeometry (private->xdisplay, xwindow, &root, &wx, &wy, &ww, &wh, &wb, &wd))
+	{
+	  if (x)
+	    *x = wx;
+	  if (y)
+	    *y = wy;
+	}
+    }
+}
+
 GdkWindow*
 gdk_window_get_pointer (GdkWindow       *window,
 			gint            *x,
@@ -1625,188 +1688,6 @@ gdk_window_shape_combine_mask (GdkWindow *window,
 #endif /* HAVE_SHAPE_EXT */
 }
 
-void
-gdk_dnd_drag_addwindow (GdkWindow *window)
-{
-  GdkWindowPrivate *window_private;
-  
-  g_return_if_fail (window != NULL);
-  
-  window_private = (GdkWindowPrivate *) window;
-  if (window_private->destroyed)
-    return;
-  
-  if (window_private->dnd_drag_enabled == 1 && gdk_dnd.drag_really == 0)
-    {
-      gdk_dnd.drag_numwindows++;
-      gdk_dnd.drag_startwindows = g_realloc (gdk_dnd.drag_startwindows,
-					     gdk_dnd.drag_numwindows
-					     * sizeof(GdkWindow *));
-      gdk_dnd.drag_startwindows[gdk_dnd.drag_numwindows - 1] = window;
-      window_private->dnd_drag_accepted = 0;
-    } 
-  else
-    g_warning ("dnd_really is 1 or drag is not enabled! can't addwindow\n");
-}
-
-void
-gdk_window_dnd_drag_set (GdkWindow   *window,
-			 guint8       drag_enable,
-			 gchar      **typelist,
-			 guint        numtypes)
-{
-  GdkWindowPrivate *window_private;
-  int i, wasset = 0;
-  
-  g_return_if_fail (window != NULL);
-  window_private = (GdkWindowPrivate *) window;
-  if (window_private->destroyed)
-    return;
-  
-  window_private->dnd_drag_enabled = drag_enable ? 1 : 0;
-  
-  if (drag_enable)
-    {
-      g_return_if_fail(typelist != NULL);
-      
-      if (window_private->dnd_drag_data_numtypesavail > 3)
-	wasset = 1;
-      window_private->dnd_drag_data_numtypesavail = numtypes;
-      
-      window_private->dnd_drag_data_typesavail =
-	g_realloc (window_private->dnd_drag_data_typesavail,
-		   (numtypes + 1) * sizeof (GdkAtom));
-      
-      for (i = 0; i < numtypes; i++)
-	{
-	  /* Allow blanket use of ALL to get anything... */
-	  if (strcmp (typelist[i], "ALL"))
-	    window_private->dnd_drag_data_typesavail[i] =
-	      gdk_atom_intern (typelist[i], FALSE);
-	  else
-	    window_private->dnd_drag_data_typesavail[i] = None;
-	}
-      
-      /* 
-       * set our extended type list if we need to 
-       */
-      if (numtypes > 3)
-	gdk_property_change(window, gdk_dnd.gdk_XdeTypelist,
-			    XA_PRIMARY, 32, GDK_PROP_MODE_REPLACE,
-			    (guchar *)(window_private->dnd_drag_data_typesavail
-			     + (sizeof(GdkAtom) * 3)),
-			    (numtypes - 3) * sizeof(GdkAtom));
-      else if (wasset)
-	gdk_property_delete (window, gdk_dnd.gdk_XdeTypelist);
-    }
-  else
-    {
-      g_free (window_private->dnd_drag_data_typesavail);
-      window_private->dnd_drag_data_typesavail = NULL;
-      window_private->dnd_drag_data_numtypesavail = 0;
-    }
-}
-
-void
-gdk_window_dnd_drop_set (GdkWindow   *window,
-			 guint8       drop_enable,
-			 gchar      **typelist,
-			 guint        numtypes,
-			 guint8       destructive_op)
-{
-  GdkWindowPrivate *window_private;
-  int i;
-  
-  g_return_if_fail (window != NULL);
-  window_private = (GdkWindowPrivate *) window;
-  if (window_private->destroyed)
-    return;
-  
-  window_private->dnd_drop_enabled = drop_enable ? 1 : 0;
-  if (drop_enable)
-    {
-      g_return_if_fail(typelist != NULL);
-      
-      window_private->dnd_drop_data_numtypesavail = numtypes;
-      
-      window_private->dnd_drop_data_typesavail =
-	g_realloc (window_private->dnd_drop_data_typesavail,
-		   (numtypes + 1) * sizeof (GdkAtom));
-      
-      for (i = 0; i < numtypes; i++)
-	window_private->dnd_drop_data_typesavail[i] =
-	  gdk_atom_intern (typelist[i], FALSE);
-      
-      window_private->dnd_drop_destructive_op = destructive_op;
-    }
-}
-
-/* 
- * This is used to reply to a GDK_DRAG_REQUEST event
- * (which may be generated by XdeRequest or a confirmed drop... 
- */
-void
-gdk_window_dnd_data_set (GdkWindow       *window,
-			 GdkEvent        *event,
-			 gpointer         data,
-			 gulong           data_numbytes)
-{
-  GdkWindowPrivate *window_private;
-  XEvent sev;
-  GdkEventDropDataAvailable tmp_ev;
-  gchar *tmp;
-  
-  g_return_if_fail (window != NULL);
-  g_return_if_fail (event != NULL);
-  g_return_if_fail (data != NULL);
-  g_return_if_fail (data_numbytes > 0);
-  g_return_if_fail (event->type == GDK_DRAG_REQUEST);
-
-  window_private = (GdkWindowPrivate *) window;
-  g_return_if_fail (window_private->dnd_drag_accepted != 0);    
-  if (window_private->destroyed)
-    return;
-  
-  /* We set the property on our window... */
-  gdk_property_change (window, window_private->dnd_drag_data_type,
-		       XA_PRIMARY, 8, GDK_PROP_MODE_REPLACE, data,
-		       data_numbytes);
-  tmp = gdk_atom_name(window_private->dnd_drag_data_type);
-#ifdef DEBUG_DND
-  g_message("DnD type %s on window %ld\n", tmp, window_private->xwindow);
-#endif
-  g_free(tmp);
-  
-  /* 
-   * Then we send the event to tell the receiving window that the
-   * drop has happened 
-   */
-  tmp_ev.u.allflags = 0;
-  tmp_ev.u.flags.protocol_version = DND_PROTOCOL_VERSION;
-  tmp_ev.u.flags.isdrop = event->dragrequest.isdrop;
-  
-  sev.xclient.type = ClientMessage;
-  sev.xclient.format = 32;
-  sev.xclient.window = event->dragrequest.requestor;
-  sev.xclient.message_type = gdk_dnd.gdk_XdeDataAvailable;
-  sev.xclient.data.l[0] = window_private->xwindow;
-  sev.xclient.data.l[1] = tmp_ev.u.allflags;
-  sev.xclient.data.l[2] = window_private->dnd_drag_data_type;
-
-  if (event->dragrequest.isdrop)
-    sev.xclient.data.l[3] = event->dragrequest.drop_coords.x +
-      (event->dragrequest.drop_coords.y << 16);
-  else
-    sev.xclient.data.l[3] = 0;
-
-  sev.xclient.data.l[4] = event->dragrequest.timestamp;
-
-  if (!gdk_send_xevent (event->dragrequest.requestor, False,
-		       StructureNotifyMask, &sev))
-    GDK_NOTE (DND, g_message("Sending XdeDataAvailable to %#x failed\n",
-			     event->dragrequest.requestor));
-}
-
 void          
 gdk_window_add_filter     (GdkWindow     *window,
 			   GdkFilterFunc  function,
@@ -1849,7 +1730,7 @@ gdk_window_remove_filter  (GdkWindow     *window,
 			   gpointer       data)
 {
   GdkWindowPrivate *private;
-  GList *tmp_list;
+  GList *tmp_list, *node;
   GdkEventFilter *filter;
 
   private = (GdkWindowPrivate*) window;
@@ -1862,15 +1743,16 @@ gdk_window_remove_filter  (GdkWindow     *window,
   while (tmp_list)
     {
       filter = (GdkEventFilter *)tmp_list->data;
+      node = tmp_list;
       tmp_list = tmp_list->next;
 
       if ((filter->function == function) && (filter->data == data))
 	{
 	  if(private)
-	    private->filters = g_list_remove_link (private->filters, tmp_list);
+	    private->filters = g_list_remove_link (private->filters, node);
 	  else
 	    gdk_default_filters = g_list_remove_link (gdk_default_filters, tmp_list);
-	  g_list_free_1 (tmp_list);
+	  g_list_free_1 (node);
 	  g_free (filter);
 	  
 	  return;
