@@ -34,10 +34,10 @@ static void gtk_adjustment_init       (GtkAdjustment      *adjustment);
 static guint adjustment_signals[LAST_SIGNAL] = { 0 };
 
 
-guint
-gtk_adjustment_get_type ()
+GtkType
+gtk_adjustment_get_type (void)
 {
-  static guint adjustment_type = 0;
+  static GtkType adjustment_type = 0;
 
   if (!adjustment_type)
     {
@@ -48,11 +48,12 @@ gtk_adjustment_get_type ()
 	sizeof (GtkAdjustmentClass),
 	(GtkClassInitFunc) gtk_adjustment_class_init,
 	(GtkObjectInitFunc) gtk_adjustment_init,
-	(GtkArgSetFunc) NULL,
-	(GtkArgGetFunc) NULL,
+	/* reserved_1 */ NULL,
+	/* reserved_2 */ NULL,
+        (GtkClassInitFunc) NULL,
       };
 
-      adjustment_type = gtk_type_unique (gtk_data_get_type (), &adjustment_info);
+      adjustment_type = gtk_type_unique (GTK_TYPE_DATA, &adjustment_info);
     }
 
   return adjustment_type;
@@ -70,14 +71,14 @@ gtk_adjustment_class_init (GtkAdjustmentClass *class)
                     GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
                     object_class->type,
                     GTK_SIGNAL_OFFSET (GtkAdjustmentClass, changed),
-                    gtk_signal_default_marshaller,
+                    gtk_marshal_NONE__NONE,
 		    GTK_TYPE_NONE, 0);
   adjustment_signals[VALUE_CHANGED] =
     gtk_signal_new ("value_changed",
                     GTK_RUN_FIRST | GTK_RUN_NO_RECURSE,
                     object_class->type,
                     GTK_SIGNAL_OFFSET (GtkAdjustmentClass, value_changed),
-                    gtk_signal_default_marshaller,
+                    gtk_marshal_NONE__NONE,
 		    GTK_TYPE_NONE, 0);
 
   gtk_object_class_add_signals (object_class, adjustment_signals, LAST_SIGNAL);
@@ -126,7 +127,60 @@ gtk_adjustment_set_value (GtkAdjustment        *adjustment,
   g_return_if_fail (adjustment != NULL);
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 
-  adjustment->value = CLAMP (value, adjustment->lower, adjustment->upper);
+  value = CLAMP (value, adjustment->lower, adjustment->upper);
 
-  gtk_signal_emit_by_name (GTK_OBJECT (adjustment), "value_changed");
+  if (value != adjustment->value)
+    {
+      adjustment->value = value;
+
+      gtk_adjustment_value_changed (adjustment);
+    }
+}
+
+void
+gtk_adjustment_changed (GtkAdjustment        *adjustment)
+{
+  g_return_if_fail (adjustment != NULL);
+  g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+
+  gtk_signal_emit (GTK_OBJECT (adjustment), adjustment_signals[CHANGED]);
+}
+
+void
+gtk_adjustment_value_changed (GtkAdjustment        *adjustment)
+{
+  g_return_if_fail (adjustment != NULL);
+  g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+
+  gtk_signal_emit (GTK_OBJECT (adjustment), adjustment_signals[VALUE_CHANGED]);
+}
+
+void
+gtk_adjustment_clamp_page (GtkAdjustment *adjustment,
+			   gfloat         lower,
+			   gfloat         upper)
+{
+  gint need_emission;
+
+  g_return_if_fail (adjustment != NULL);
+  g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+
+  lower = CLAMP (lower, adjustment->lower, adjustment->upper);
+  upper = CLAMP (upper, adjustment->lower, adjustment->upper);
+
+  need_emission = FALSE;
+
+  if (adjustment->value + adjustment->page_size < upper)
+    {
+      adjustment->value = upper - adjustment->page_size;
+      need_emission = TRUE;
+    }
+  if (adjustment->value > lower)
+    {
+      adjustment->value = lower;
+      need_emission = TRUE;
+    }
+
+  if (need_emission)
+    gtk_adjustment_value_changed (adjustment);
 }
