@@ -29,10 +29,10 @@ static gint default_child_ipad_x = 7;
 static gint default_child_ipad_y = 0;
 
 
-guint
+GtkType
 gtk_button_box_get_type (void)
 {
-  static guint button_box_type = 0;
+  static GtkType button_box_type = 0;
 
   if (!button_box_type)
     {
@@ -43,8 +43,9 @@ gtk_button_box_get_type (void)
 	sizeof (GtkButtonBoxClass),
 	(GtkClassInitFunc) gtk_button_box_class_init,
 	(GtkObjectInitFunc) gtk_button_box_init,
-	(GtkArgSetFunc) NULL,
-        (GtkArgGetFunc) NULL,
+	/* reserved_1 */ NULL,
+        /* reserved_2 */ NULL,
+        (GtkClassInitFunc) NULL,
       };
 
       button_box_type = gtk_type_unique (gtk_box_get_type (), &button_box_info);
