@@ -31,21 +31,21 @@
 
 #ifdef __cplusplus
 extern "C" {
-#pragma }
 #endif /* __cplusplus */
 
 /* clist flags */
 enum                    
 {
-  GTK_CLIST_FROZEN          = 1 << 0,                                     
-  GTK_CLIST_IN_DRAG         = 1 << 1,                                        
-  GTK_CLIST_DRAG_SELECTION  = 1 << 2,
-  GTK_CLIST_ROW_HEIGHT_SET  = 1 << 3,
-  GTK_CLIST_SHOW_TITLES     = 1 << 4,
-  GTK_CLIST_CONSTRUCTED	    = 1 << 5,
-  GTK_CLIST_CHILD_HAS_FOCUS = 1 << 6,
-  GTK_CLIST_ADD_MODE        = 1 << 7,
-  GTK_CLIST_AUTO_SORT       = 1 << 8
+  GTK_CLIST_FROZEN              = 1 << 0,
+  GTK_CLIST_IN_DRAG             = 1 << 1,
+  GTK_CLIST_DRAG_SELECTION      = 1 << 2,
+  GTK_CLIST_ROW_HEIGHT_SET      = 1 << 3,
+  GTK_CLIST_SHOW_TITLES         = 1 << 4,
+  GTK_CLIST_CONSTRUCTED	        = 1 << 5,
+  GTK_CLIST_CHILD_HAS_FOCUS     = 1 << 6,
+  GTK_CLIST_ADD_MODE            = 1 << 7,
+  GTK_CLIST_AUTO_SORT           = 1 << 8,
+  GTK_CLIST_AUTO_RESIZE_BLOCKED = 1 << 9
 }; 
 
 /* cell types */
@@ -77,6 +77,7 @@ typedef enum
 #define GTK_CLIST_DRAG_SELECTION(clist)    (GTK_CLIST_FLAGS (clist) & GTK_CLIST_DRAG_SELECTION)
 #define GTK_CLIST_ADD_MODE(clist)          (GTK_CLIST_FLAGS (clist) & GTK_CLIST_ADD_MODE)
 #define GTK_CLIST_AUTO_SORT(clist)         (GTK_CLIST_FLAGS (clist) & GTK_CLIST_AUTO_SORT)
+#define GTK_CLIST_AUTO_RESIZE_BLOCKED(clist) (GTK_CLIST_FLAGS (clist) & GTK_CLIST_AUTO_RESIZE_BLOCKED)
 
 #define GTK_CLIST_ROW(_glist_) ((GtkCListRow *)((_glist_)->data))
 
@@ -192,61 +193,68 @@ struct _GtkCListClass
 {
   GtkContainerClass parent_class;
   
-  void   (*select_row)          (GtkCList     *clist,
-				 gint          row,
-				 gint          column,
-				 GdkEvent     *event);
-  void   (*unselect_row)        (GtkCList     *clist,
-				 gint          row,
-				 gint          column,
-				 GdkEvent     *event);
-  void   (*click_column)        (GtkCList     *clist,
-				 gint          column);
-  void   (*toggle_focus_row)    (GtkCList     *clist);
-  void   (*select_all)          (GtkCList     *clist);
-  void   (*unselect_all)        (GtkCList     *clist);
-  void   (*undo_selection)      (GtkCList     *clist);
-  void   (*start_selection)     (GtkCList     *clist);
-  void   (*end_selection)       (GtkCList     *clist);
-  void   (*extend_selection)    (GtkCList     *clist,
-				 GtkScrollType scroll_type,
-				 gfloat        position,
-				 gboolean      auto_start_selection);
-  void   (*scroll_horizontal)   (GtkCList     *clist,
-				 GtkScrollType scroll_type,
-				 gfloat        position);
-  void   (*scroll_vertical)     (GtkCList     *clist,
-				 GtkScrollType scroll_type,
-				 gfloat        position);
-  void   (*toggle_add_mode)     (GtkCList     *clist);
-  void   (*abort_column_resize) (GtkCList     *clist);
-  void   (*resync_selection)    (GtkCList     *clist,
-				 GdkEvent     *event);
-  GList* (*selection_find)      (GtkCList     *clist,
-				 gint          row_number,
-				 GList        *row_list_element);
-  void   (*draw_row)            (GtkCList     *clist,
-				 GdkRectangle *area,
-				 gint          row,
-				 GtkCListRow  *clist_row);
-  void   (*clear)               (GtkCList     *clist);
-  void   (*fake_unselect_all)   (GtkCList     *clist,
-				 gint          row);
-  void   (*sort_list)           (GtkCList     *clist);
-  gint   (*insert_row)          (GtkCList     *clist,
-				 gint          row,
-				 gchar       *text[]);
-  void   (*remove_row)          (GtkCList    *clist,
-				 gint         row);
-  void   (*set_cell_contents)   (GtkCList    *clist,
-				 GtkCListRow *clist_row,
-				 gint         column,
-				 GtkCellType  type,
-				 const gchar *text,
-				 guint8       spacing,
-				 GdkPixmap   *pixmap,
-				 GdkBitmap   *mask);
-  
+  void   (*select_row)          (GtkCList       *clist,
+				 gint            row,
+				 gint            column,
+				 GdkEvent       *event);
+  void   (*unselect_row)        (GtkCList       *clist,
+				 gint            row,
+				 gint            column,
+				 GdkEvent       *event);
+  void   (*click_column)        (GtkCList       *clist,
+				 gint            column);
+  void   (*resize_column)       (GtkCList       *clist,
+				 gint            column,
+                                 gint            width);
+  void   (*toggle_focus_row)    (GtkCList       *clist);
+  void   (*select_all)          (GtkCList       *clist);
+  void   (*unselect_all)        (GtkCList       *clist);
+  void   (*undo_selection)      (GtkCList       *clist);
+  void   (*start_selection)     (GtkCList       *clist);
+  void   (*end_selection)       (GtkCList       *clist);
+  void   (*extend_selection)    (GtkCList       *clist,
+				 GtkScrollType   scroll_type,
+				 gfloat          position,
+				 gboolean        auto_start_selection);
+  void   (*scroll_horizontal)   (GtkCList       *clist,
+				 GtkScrollType   scroll_type,
+				 gfloat          position);
+  void   (*scroll_vertical)     (GtkCList       *clist,
+				 GtkScrollType   scroll_type,
+				 gfloat          position);
+  void   (*toggle_add_mode)     (GtkCList       *clist);
+  void   (*abort_column_resize) (GtkCList       *clist);
+  void   (*resync_selection)    (GtkCList       *clist,
+				 GdkEvent       *event);
+  GList* (*selection_find)      (GtkCList       *clist,
+				 gint            row_number,
+				 GList          *row_list_element);
+  void   (*draw_row)            (GtkCList       *clist,
+				 GdkRectangle   *area,
+				 gint            row,
+				 GtkCListRow    *clist_row);
+  void   (*clear)               (GtkCList       *clist);
+  void   (*fake_unselect_all)   (GtkCList       *clist,
+				 gint            row);
+  void   (*sort_list)           (GtkCList       *clist);
+  gint   (*insert_row)          (GtkCList       *clist,
+				 gint            row,
+				 gchar          *text[]);
+  void   (*remove_row)          (GtkCList       *clist,
+				 gint            row);
+  void   (*set_cell_contents)   (GtkCList       *clist,
+				 GtkCListRow    *clist_row,
+				 gint            column,
+				 GtkCellType     type,
+				 const gchar    *text,
+				 guint8          spacing,
+				 GdkPixmap      *pixmap,
+				 GdkBitmap      *mask);
+  void   (*cell_size_request)   (GtkCList       *clist,
+				 GtkCListRow    *clist_row,
+				 gint            column,
+				 GtkRequisition *requisition);
+
   gint scrollbar_spacing;
 };
 
@@ -259,9 +267,14 @@ struct _GtkCListColumn
   GdkWindow *window;
   
   gint width;
+  gint min_width;
+  gint max_width;
   GtkJustification justification;
   
-  gint width_set : 1;
+  guint visible     : 1;  
+  guint width_set   : 1;
+  guint resizeable  : 1;
+  guint auto_resize : 1;
 };
 
 struct _GtkCListRow
@@ -272,12 +285,14 @@ struct _GtkCListRow
   GdkColor foreground;
   GdkColor background;
   
+  GtkStyle *style;
+
   gpointer data;
   GtkDestroyNotify destroy;
   
-  gint fg_set     : 1;
-  gint bg_set     : 1;
-  gint selectable : 1;
+  guint fg_set     : 1;
+  guint bg_set     : 1;
+  guint selectable : 1;
 };
 
 /* Cell Structures */
@@ -285,9 +300,11 @@ struct _GtkCellText
 {
   GtkCellType type;
   
-  gint vertical;
-  gint horizontal;
+  gint16 vertical;
+  gint16 horizontal;
   
+  GtkStyle *style;
+
   gchar *text;
 };
 
@@ -295,9 +312,11 @@ struct _GtkCellPixmap
 {
   GtkCellType type;
   
-  gint vertical;
-  gint horizontal;
+  gint16 vertical;
+  gint16 horizontal;
   
+  GtkStyle *style;
+
   GdkPixmap *pixmap;
   GdkBitmap *mask;
 };
@@ -306,9 +325,11 @@ struct _GtkCellPixText
 {
   GtkCellType type;
   
-  gint vertical;
-  gint horizontal;
+  gint16 vertical;
+  gint16 horizontal;
   
+  GtkStyle *style;
+
   gchar *text;
   guint8 spacing;
   GdkPixmap *pixmap;
@@ -319,9 +340,11 @@ struct _GtkCellWidget
 {
   GtkCellType type;
   
-  gint vertical;
-  gint horizontal;
+  gint16 vertical;
+  gint16 horizontal;
   
+  GtkStyle *style;
+
   GtkWidget *widget;
 };
 
@@ -329,9 +352,11 @@ struct _GtkCell
 {
   GtkCellType type;
   
-  gint vertical;
-  gint horizontal;
+  gint16 vertical;
+  gint16 horizontal;
   
+  GtkStyle *style;
+
   union {
     gchar *text;
     
@@ -356,32 +381,36 @@ GtkType gtk_clist_get_type (void);
 /* constructers useful for gtk-- wrappers */
 void gtk_clist_construct (GtkCList *clist,
 			  gint      columns,
-			  gchar   *titles[]);
+			  gchar    *titles[]);
 
 /* create a new GtkCList */
 GtkWidget *gtk_clist_new             (gint   columns);
 GtkWidget *gtk_clist_new_with_titles (gint   columns,
 				      gchar *titles[]);
 
+/* deprecated function, use gtk_clist_set_shadow_type instead. */
+void gtk_clist_set_border (GtkCList      *clist,
+			   GtkShadowType  border);
+
 /* set the border style of the clist */
-void gtk_clist_set_border (GtkCList     *clist,
-			   GtkShadowType border);
+void gtk_clist_set_shadow_type (GtkCList      *clist,
+				GtkShadowType  type);
 
 /* set the clist's selection mode */
-void gtk_clist_set_selection_mode (GtkCList        *clist,
-				   GtkSelectionMode mode);
+void gtk_clist_set_selection_mode (GtkCList         *clist,
+				   GtkSelectionMode  mode);
 
 /* set policy on the scrollbar, to either show them all the time
- * or show them only when they are needed, ie., when there is more than one page
- * of information
+ * or show them only when they are needed, ie., when there is more
+ * than one page of information
  */
-void gtk_clist_set_policy (GtkCList     *clist,
-			   GtkPolicyType vscrollbar_policy,
-			   GtkPolicyType hscrollbar_policy);
+void gtk_clist_set_policy (GtkCList      *clist,
+			   GtkPolicyType  vscrollbar_policy,
+			   GtkPolicyType  hscrollbar_policy);
 
-/* freeze all visual updates of the list, and then thaw the list after you have made
- * a number of changes and the updates wil occure in a more efficent mannor than if
- * you made them on a unfrozen list
+/* freeze all visual updates of the list, and then thaw the list after
+ * you have made a number of changes and the updates wil occure in a
+ * more efficent mannor than if you made them on a unfrozen list
  */
 void gtk_clist_freeze (GtkCList *clist);
 void gtk_clist_thaw   (GtkCList *clist);
@@ -412,9 +441,28 @@ void gtk_clist_set_column_widget (GtkCList  *clist,
 				  GtkWidget *widget);
 
 /* set the justification on a column */
-void gtk_clist_set_column_justification (GtkCList        *clist,
-					 gint             column,
-					 GtkJustification justification);
+void gtk_clist_set_column_justification (GtkCList         *clist,
+					 gint              column,
+					 GtkJustification  justification);
+
+/* set visibility of a column */
+void gtk_clist_set_column_visibility (GtkCList *clist,
+				      gint      column,
+				      gboolean  visible);
+
+/* enable/disable column resize operations by mouse */
+void gtk_clist_set_column_resizeable (GtkCList *clist,
+				      gint      column,
+				      gboolean  resizeable);
+
+/* resize column automatically to its optimal width */
+void gtk_clist_set_column_auto_resize (GtkCList *clist,
+				       gint      column,
+				       gboolean  auto_resize);
+
+/* return the optimal column width, i.e. maximum of all cell widths */
+gint gtk_clist_optimal_column_width (GtkCList *clist,
+				     gint      column);
 
 /* set the pixel width of a column; this is a necessary step in
  * creating a CList because otherwise the column width is chozen from
@@ -424,17 +472,24 @@ void gtk_clist_set_column_width (GtkCList *clist,
 				 gint      column,
 				 gint      width);
 
+/* set column minimum/maximum width. min/max_width < 0 => no restriction */
+void gtk_clist_set_column_min_width (GtkCList *clist,
+				     gint      column,
+				     gint      min_width);
+void gtk_clist_set_column_max_width (GtkCList *clist,
+				     gint      column,
+				     gint      max_width);
+
 /* change the height of the rows, the default is the hight
  * of the current font
  */
 void gtk_clist_set_row_height (GtkCList *clist,
 			       gint      height);
 
-/* scroll the viewing area of the list to the given column
- * and row; row_align and col_align are between 0-1 representing the
- * location the row should appear on the screnn, 0.0 being top or left,
- * 1.0 being bottom or right; if row or column is -1 then then there
- * is no change
+/* scroll the viewing area of the list to the given column and row;
+ * row_align and col_align are between 0-1 representing the location the
+ * row should appear on the screnn, 0.0 being top or left, 1.0 being
+ * bottom or right; if row or column is -1 then then there is no change
  */
 void gtk_clist_moveto (GtkCList *clist,
 		       gint      row,
@@ -460,10 +515,10 @@ void gtk_clist_set_text (GtkCList    *clist,
 /* for the "get" functions, any of the return pointer can be
  * NULL if you are not interested
  */
-gint gtk_clist_get_text (GtkCList *clist,
-			 gint      row,
-			 gint      column,
-			 gchar   **text);
+gint gtk_clist_get_text (GtkCList  *clist,
+			 gint       row,
+			 gint       column,
+			 gchar    **text);
 
 /* sets a given cell's pixmap, replacing it's current contents */
 void gtk_clist_set_pixmap (GtkCList  *clist,
@@ -487,13 +542,13 @@ void gtk_clist_set_pixtext (GtkCList    *clist,
 			    GdkPixmap   *pixmap,
 			    GdkBitmap   *mask);
 
-gint gtk_clist_get_pixtext (GtkCList    *clist,
-			    gint         row,
-			    gint         column,
-			    gchar      **text,
-			    guint8      *spacing,
-			    GdkPixmap  **pixmap,
-			    GdkBitmap  **mask);
+gint gtk_clist_get_pixtext (GtkCList   *clist,
+			    gint        row,
+			    gint        column,
+			    gchar     **text,
+			    guint8     *spacing,
+			    GdkPixmap **pixmap,
+			    GdkBitmap **mask);
 
 /* sets the foreground color of a row, the colar must already
  * be allocated
@@ -509,9 +564,26 @@ void gtk_clist_set_background (GtkCList *clist,
 			       gint      row,
 			       GdkColor *color);
 
+/* set / get cell styles */
+void gtk_clist_set_cell_style (GtkCList *clist,
+			       gint      row,
+			       gint      column,
+			       GtkStyle *style);
+
+GtkStyle *gtk_clist_get_cell_style (GtkCList *clist,
+				    gint      row,
+				    gint      column);
+
+void gtk_clist_set_row_style (GtkCList *clist,
+			      gint      row,
+			      GtkStyle *style);
+
+GtkStyle *gtk_clist_get_row_style (GtkCList *clist,
+				   gint      row);
+
 /* this sets a horizontal and vertical shift for drawing
- * the contents of a cell; it can be positive or negitive; this is
- * particulary useful for indenting items in a column
+ * the contents of a cell; it can be positive or negitive;
+ * this is particulary useful for indenting items in a column
  */
 void gtk_clist_set_shift (GtkCList *clist,
 			  gint      row,
@@ -526,14 +598,16 @@ void gtk_clist_set_selectable (GtkCList *clist,
 gboolean gtk_clist_get_selectable (GtkCList *clist,
 				   gint      row);
 
-/* append returns the index of the row you just added, making
- * it easier to append and modify a row
+/* prepend/append returns the index of the row you just added,
+ * making it easier to append and modify a row
  */
-gint gtk_clist_append (GtkCList *clist,
-		       gchar    *text[]);
+gint gtk_clist_prepend (GtkCList *clist,
+		        gchar    *text[]);
+gint gtk_clist_append  (GtkCList *clist,
+			gchar    *text[]);
 
-/* inserts a row at index row and returns the row where it was actually
- * inserted (may be different from "row" in auto_sort mode)
+/* inserts a row at index row and returns the row where it was
+ * actually inserted (may be different from "row" in auto_sort mode)
  */
 gint gtk_clist_insert (GtkCList *clist,
 		       gint      row,
@@ -549,10 +623,10 @@ void gtk_clist_set_row_data (GtkCList *clist,
 			     gpointer  data);
 
 /* sets a data pointer for a given row with destroy notification */
-void gtk_clist_set_row_data_full (GtkCList        *clist,
-			          gint             row,
-			          gpointer         data,
-				  GtkDestroyNotify destroy);
+void gtk_clist_set_row_data_full (GtkCList         *clist,
+			          gint              row,
+			          gpointer          data,
+				  GtkDestroyNotify  destroy);
 
 /* returns the data set for a row */
 gpointer gtk_clist_get_row_data (GtkCList *clist,
@@ -577,12 +651,15 @@ void gtk_clist_unselect_row (GtkCList *clist,
 /* undo the last select/unselect operation */
 void gtk_clist_undo_selection (GtkCList *clist);
 
-/* clear the entire list -- this is much faster than removing each item 
- * with gtk_clist_remove
+/* clear the entire list -- this is much faster than removing
+ * each item with gtk_clist_remove
  */
 void gtk_clist_clear (GtkCList *clist);
 
-/* return the row column corresponding to the x and y coordinates */
+/* return the row column corresponding to the x and y coordinates,
+ * the returned values are only valid if the x and y coordinates
+ * are respectively to a window == clist->clist_window
+ */
 gint gtk_clist_get_selection_info (GtkCList *clist,
 			     	   gint      x,
 			     	   gint      y,
@@ -609,8 +686,8 @@ void gtk_clist_set_sort_column (GtkCList *clist,
 				gint      column);
 
 /* how to sort : ascending or descending */
-void gtk_clist_set_sort_type (GtkCList   *clist,
-			      GtkSortType sort_type);
+void gtk_clist_set_sort_type (GtkCList    *clist,
+			      GtkSortType  sort_type);
 
 /* sort the list with the current compare function */
 void gtk_clist_sort (GtkCList *clist);
