@@ -13,8 +13,9 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /* The GtkHandleBox is to allow widgets to be dragged in and out of
@@ -54,7 +55,6 @@ struct _GtkHandleBox
   guint		  child_detached : 1;
   guint		  in_drag : 1;
   guint		  shrink_on_detach : 1;
-  GdkCursor      *fleur_cursor;
 
   gint dragoff_x, dragoff_y; /* start drag position (wrt widget->window) */
 };
