@@ -50,8 +50,9 @@ gtk_hpaned_get_type (void)
 	sizeof (GtkHPanedClass),
 	(GtkClassInitFunc) gtk_hpaned_class_init,
 	(GtkObjectInitFunc) gtk_hpaned_init,
-	(GtkArgSetFunc) NULL,
-        (GtkArgGetFunc) NULL,
+	/* reserved_1 */ NULL,
+        /* reserved_2 */ NULL,
+        (GtkClassInitFunc) NULL,
       };
 
       hpaned_type = gtk_type_unique (gtk_paned_get_type (), &hpaned_info);
@@ -261,10 +262,12 @@ gtk_hpaned_xor_line (GtkPaned *paned)
 
   if (!paned->xor_gc)
     {
-      values.function = GDK_INVERT;
+      values.foreground = widget->style->white;
+      values.function = GDK_XOR;
       values.subwindow_mode = GDK_INCLUDE_INFERIORS;
       paned->xor_gc = gdk_gc_new_with_values (widget->window,
 					      &values,
+					      GDK_GC_FOREGROUND |
 					      GDK_GC_FUNCTION |
 					      GDK_GC_SUBWINDOW);
     }
