@@ -18,14 +18,26 @@
  */
 #include <string.h>
 #include "gtklabel.h"
-
+#include "gdk/gdkkeysyms.h"
 
 enum {
   ARG_0,
   ARG_LABEL,
+  ARG_PATTERN,
   ARG_JUSTIFY
 };
 
+typedef struct _GtkLabelRow GtkLabelRow;
+
+struct _GtkLabelRow {
+  gint index;
+  gint width;
+  gint height;
+  gint len;
+};
+
+GMemChunk *row_mem_chunk = NULL;
+
 static void gtk_label_class_init   (GtkLabelClass  *klass);
 static void gtk_label_init	   (GtkLabel	   *label);
 static void gtk_label_set_arg	   (GtkObject	   *object,
@@ -43,6 +55,7 @@ static void gtk_label_state_changed (GtkWidget	    *widget,
 				     guint	     previous_state);
 static void gtk_label_style_set	    (GtkWidget	    *widget,
 				     GtkStyle	    *previous_style);
+static void gtk_label_free_rows     (GtkLabel       *label);
 
 
 
@@ -87,6 +100,7 @@ gtk_label_class_init (GtkLabelClass *class)
   parent_class = gtk_type_class (gtk_misc_get_type ());
   
   gtk_object_add_arg_type ("GtkLabel::label", GTK_TYPE_STRING, GTK_ARG_READWRITE, ARG_LABEL);
+  gtk_object_add_arg_type ("GtkLabel::pattern", GTK_TYPE_STRING, GTK_ARG_READWRITE, ARG_PATTERN);
   gtk_object_add_arg_type ("GtkLabel::justify", GTK_TYPE_JUSTIFICATION, GTK_ARG_READWRITE, ARG_JUSTIFY);
 
   object_class->set_arg = gtk_label_set_arg;
@@ -113,6 +127,9 @@ gtk_label_set_arg (GtkObject	  *object,
     case ARG_LABEL:
       gtk_label_set (label, GTK_VALUE_STRING (*arg) ? GTK_VALUE_STRING (*arg) : "");
       break;
+    case ARG_PATTERN:
+      gtk_label_set_pattern (label, GTK_VALUE_STRING (*arg));
+      break;
     case ARG_JUSTIFY:
       gtk_label_set_justify (label, GTK_VALUE_ENUM (*arg));
       break;
@@ -135,6 +152,9 @@ gtk_label_get_arg (GtkObject	  *object,
     case ARG_LABEL:
       GTK_VALUE_STRING (*arg) = g_strdup (label->label);
       break;
+    case ARG_PATTERN:
+      GTK_VALUE_STRING (*arg) = g_strdup (label->pattern);
+      break;
     case ARG_JUSTIFY:
       GTK_VALUE_ENUM (*arg) = label->jtype;
       break;
@@ -154,6 +174,7 @@ gtk_label_init (GtkLabel *label)
   label->max_width = 0;
   label->jtype = GTK_JUSTIFY_CENTER;
   label->needs_clear = FALSE;
+  label->pattern = NULL;
   
   gtk_label_set (label, "");
 }
@@ -181,18 +202,57 @@ gtk_label_set (GtkLabel	   *label,
   g_return_if_fail (label != NULL);
   g_return_if_fail (GTK_IS_LABEL (label));
   g_return_if_fail (str != NULL);
+
+  if (!row_mem_chunk)
+    row_mem_chunk = g_mem_chunk_create (GtkLabelRow, 64, G_ALLOC_AND_FREE);
   
   if (label->label)
     g_free (label->label);
   label->label = g_strdup (str);
   
   if (label->row)
-    g_slist_free (label->row);
-  label->row = NULL;
-  label->row = g_slist_append (label->row, label->label);
+    gtk_label_free_rows (label);
+
   p = label->label;
-  while ((p = strchr(p, '\n')))
-    label->row = g_slist_append (label->row, ++p);
+  do {
+    GtkLabelRow *row = g_chunk_new (GtkLabelRow, row_mem_chunk);
+    label->row = g_slist_append (label->row, row);
+
+    row->index = p - label->label;
+    
+    p = strchr(p, '\n');
+    if (p)
+      {
+	p++;
+	row->len = (p - label->label) - row->index;
+      }
+    else
+      row->len = strlen (label->label) - row->index;
+  } while (p);
+
+  if (GTK_WIDGET_VISIBLE (label))
+    {
+      if (GTK_WIDGET_MAPPED (label))
+	gdk_window_clear_area (GTK_WIDGET (label)->window,
+			       GTK_WIDGET (label)->allocation.x,
+			       GTK_WIDGET (label)->allocation.y,
+			       GTK_WIDGET (label)->allocation.width,
+			       GTK_WIDGET (label)->allocation.height);
+      
+      gtk_widget_queue_resize (GTK_WIDGET (label));
+    }
+}
+
+void
+gtk_label_set_pattern (GtkLabel	   *label,
+		       const gchar *pattern)
+{
+  g_return_if_fail (label != NULL);
+  g_return_if_fail (GTK_IS_LABEL (label));
+  
+  if (label->pattern)
+    g_free (label->pattern);
+  label->pattern = g_strdup (pattern);
   
   if (GTK_WIDGET_VISIBLE (label))
     {
@@ -255,17 +315,129 @@ gtk_label_finalize (GtkObject *object)
   label = GTK_LABEL (object);
   
   g_free (label->label);
-  g_slist_free (label->row);
+  if (label->pattern) g_free (label->pattern);
+  gtk_label_free_rows (label);
   
   (* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
+static gint
+gtk_label_process_row (GtkLabel    *label,
+		       GtkLabelRow *row,
+		       gint         x, gint y,
+		       gboolean     draw)
+{
+  GtkWidget *widget = GTK_WIDGET (label);
+  
+  char lastc;
+  gint i, j;
+  gint offset;
+  gint height;
+  gint pattern_length;
+
+  if (label->pattern) 
+    pattern_length = strlen (label->pattern);
+  else
+    pattern_length = 0;
+
+  offset = 0;
+  height = widget->style->font->ascent + widget->style->font->descent;
+  
+  if (draw)
+    {
+      if (label->jtype == GTK_JUSTIFY_CENTER)
+	offset = (label->max_width - row->width) / 2;
+      else if (label->jtype == GTK_JUSTIFY_RIGHT)
+	offset = (label->max_width - row->width);
+    }
+      
+  if (label->pattern && (row->index < pattern_length))
+    lastc = label->pattern[0];
+  else
+    lastc = ' ';
+  
+  j = 0;
+  for (i=1; i<=row->len; i++)
+    {
+      char c;
+
+      if (label->pattern && (row->index + i < pattern_length))
+	c = label->pattern[row->index+i];
+      else
+	c = ' ';
+
+      if ((i == row->len) || (lastc != c))
+	{
+	  gint width = 0;
+
+	  if (lastc == '_')
+	    {
+	      gint descent;
+	      gint rbearing;
+	      gint lbearing;
+
+	      gdk_text_extents (widget->style->font,
+				&label->label[row->index+j], i - j,
+				&lbearing, &rbearing, &width, NULL, &descent);
+
+	      if (draw)
+		{
+		  if (widget->state == GTK_STATE_INSENSITIVE)
+		    gdk_draw_line (widget->window,
+				   widget->style->white_gc,
+				   offset + x + lbearing, y + descent + 2, 
+				   offset + x + rbearing + 1, y + descent + 2);
+	      
+		  gdk_draw_line (widget->window,
+				 widget->style->fg_gc[widget->state],
+				 offset + x + lbearing - 1, y + descent + 1, 
+				 offset + x + rbearing, y + descent + 1);
+		}
+
+	      height = MAX (height, 
+			    widget->style->font->ascent + descent + 2);
+	    }
+	  else if (i != row->len)
+	    {
+	      width = gdk_text_width (widget->style->font, 
+				      &label->label[row->index+j], 
+				      i - j);
+	    }
+
+	  if (draw)
+	    {
+	      if (widget->state == GTK_STATE_INSENSITIVE)
+		gdk_draw_text (widget->window, widget->style->font,
+			       widget->style->white_gc,
+			       offset + x + 1, y + 1,
+			       &label->label[row->index+j], i - j);
+	      
+	      gdk_draw_text (widget->window, widget->style->font,
+			     widget->style->fg_gc[widget->state],
+			     offset + x, y, 
+			     &label->label[row->index+j], i - j);
+	    }
+
+		  
+	  offset += width;
+
+	  if (i != row->len)
+	    {
+	      lastc = c;
+	      j = i;
+	    }
+	}
+    }
+
+  return height;
+}
+
 static void
 gtk_label_size_request (GtkWidget      *widget,
 			GtkRequisition *requisition)
 {
   GtkLabel *label;
-  GSList *row;
+  GSList *tmp_list;
   gint width;
   
   g_return_if_fail (widget != NULL);
@@ -274,38 +446,35 @@ gtk_label_size_request (GtkWidget      *widget,
   
   label = GTK_LABEL (widget);
   
-  row = label->row;
+  requisition->height = label->misc.ypad * 2;
+
+  tmp_list = label->row;
   width = 0;
-  while (row)
+  while (tmp_list)
     {
-      if (row->next)
-	width = MAX (width,
-		     gdk_text_width (GTK_WIDGET (label)->style->font,
-				     row->data,
-				     (gchar*) row->next->data - (gchar*) row->data - 1));
-      else
-	width = MAX (width, gdk_string_width (GTK_WIDGET (label)->style->font, row->data));
-      row = row->next;
+      GtkLabelRow *row = tmp_list->data;
+
+      row->width = gdk_text_width (GTK_WIDGET (label)->style->font,
+				   &label->label [row->index],
+				   row->len);
+      width = MAX (width, row->width);
+
+      requisition->height += gtk_label_process_row (label, row, 0, 0, FALSE) + 2;
+
+      tmp_list = tmp_list->next;
     }
 
   label->max_width = width;
   requisition->width = width + label->misc.xpad * 2;
-  requisition->height = ((GTK_WIDGET (label)->style->font->ascent +
-			  GTK_WIDGET (label)->style->font->descent + 2) *
-			 g_slist_length(label->row) +
-			 label->misc.ypad * 2);
 }
-
+  
 static gint
 gtk_label_expose (GtkWidget	 *widget,
 		  GdkEventExpose *event)
 {
   GtkLabel *label;
   GtkMisc *misc;
-  GSList *row;
-  gint state;
-  gint offset;
-  gint len;
+  GSList *tmp_list;
   gint x, y;
   
   g_return_val_if_fail (widget != NULL, FALSE);
@@ -317,13 +486,11 @@ gtk_label_expose (GtkWidget	 *widget,
       label = GTK_LABEL (widget);
       misc = GTK_MISC (widget);
       
-      state = widget->state;
-      
       /*
        * GC Clipping
        */
       gdk_gc_set_clip_rectangle (widget->style->white_gc, &event->area);
-      gdk_gc_set_clip_rectangle (widget->style->fg_gc[state], &event->area);
+      gdk_gc_set_clip_rectangle (widget->style->fg_gc[widget->state], &event->area);
       
       /* We clear the whole allocation here so that if a partial
        * expose is triggered we don't just clear part and mess up
@@ -351,59 +518,41 @@ gtk_label_expose (GtkWidget	 *widget,
 	    (widget->requisition.height - misc->ypad * 2)) *
 	   misc->yalign + widget->style->font->ascent) + 1.5;
       
-      row = label->row;
-      while (row && row->next)
-	{
-	  len = (gchar*) row->next->data - (gchar*) row->data - 1;
-	  offset = 0;
-	  
-	  if (label->jtype == GTK_JUSTIFY_CENTER)
-	    offset = (label->max_width - gdk_text_width (widget->style->font, row->data, len)) / 2;
-	  
-	  else if (label->jtype == GTK_JUSTIFY_RIGHT)
-	    offset = (label->max_width - gdk_text_width (widget->style->font, row->data, len));
-	  
-	  if (state == GTK_STATE_INSENSITIVE)
-	    gdk_draw_text (widget->window, widget->style->font,
-			   widget->style->white_gc,
-			   offset + x + 1, y + 1, row->data, len);
-	  
-	  gdk_draw_text (widget->window, widget->style->font,
-			 widget->style->fg_gc[state],
-			 offset + x, y, row->data, len);
-	  row = row->next;
-	  y += widget->style->font->ascent + widget->style->font->descent + 2;
-	}
-      
       /* 
        * COMMENT: we can avoid gdk_text_width() calls here storing in label->row
        * the widths of the rows calculated in gtk_label_set.
        * Once we have a wrapping interface we can support GTK_JUSTIFY_FILL.
        */
-      offset = 0;
-      
-      if (label->jtype == GTK_JUSTIFY_CENTER)
-	offset = (label->max_width - gdk_string_width (widget->style->font, row->data)) / 2;
-      
-      else if (label->jtype == GTK_JUSTIFY_RIGHT)
-	offset = (label->max_width - gdk_string_width (widget->style->font, row->data));
-      
-      if (state == GTK_STATE_INSENSITIVE)
-	gdk_draw_string (widget->window, widget->style->font,
-			 widget->style->white_gc,
-			 offset + x + 1, y + 1, row->data);
-      
-      gdk_draw_string (widget->window, widget->style->font,
-		       widget->style->fg_gc[state],
-		       offset + x, y, row->data);
+
+      tmp_list = label->row;
+      while (tmp_list)
+	{
+	  y += gtk_label_process_row (label, tmp_list->data, x, y, TRUE) + 2;
+	  tmp_list = tmp_list->next;
+	}
       
       gdk_gc_set_clip_mask (widget->style->white_gc, NULL);
-      gdk_gc_set_clip_mask (widget->style->fg_gc[state], NULL);
+      gdk_gc_set_clip_mask (widget->style->fg_gc[widget->state], NULL);
       
     }
   return TRUE;
 }
 
+static void
+gtk_label_free_rows (GtkLabel *label)
+{
+  GSList *tmp_list;
+
+  tmp_list = label->row;
+  while (tmp_list)
+    {
+      g_mem_chunk_free (row_mem_chunk, tmp_list->data);
+      tmp_list = tmp_list->next;
+    }
+  g_slist_free (label->row);
+  label->row = NULL;
+}
+
 static void 
 gtk_label_state_changed (GtkWidget	*widget,
 			 guint		 previous_state)
@@ -420,3 +569,67 @@ gtk_label_style_set (GtkWidget	*widget,
     GTK_LABEL (widget)->needs_clear = TRUE;
 }
 
+guint      
+gtk_label_parse_uline (GtkLabel         *label,
+		       const gchar      *string)
+{
+  guint accel_key = GDK_VoidSymbol;
+  const gchar *p;
+  gchar *q, *r;
+  gchar *name, *pattern;
+
+  gint length;
+  gboolean underscore;
+
+  length = strlen (string);
+  
+  name = g_new (gchar, length+1);
+  pattern = g_new (gchar, length+1);
+
+  underscore = FALSE;
+
+  p = string;
+  q = name;
+  r = pattern;
+
+  while (*p)
+    {
+      if (underscore)
+	{
+	  if (*p == '_')
+	    *r++ = ' ';
+	  else
+	    {
+	      *r++ = '_';
+	      if (accel_key == GDK_VoidSymbol)
+		accel_key = gdk_keyval_to_lower (*p);
+	    }
+
+	  *q++ = *p;
+	  underscore = FALSE;
+	}
+      else
+	{
+	  if (*p == '_')
+	    underscore = TRUE;
+	  else
+	    {
+	      *q++ = *p;
+	      *r++ = ' ';
+	    }
+	}
+      p++;
+    }
+  *q = 0;
+  *r = 0;
+ 
+  gtk_label_set (label, name);
+  gtk_label_set_pattern (label, pattern);
+  
+  g_free (name);
+  g_free (pattern);
+
+  return accel_key;
+}
+
+
