@@ -8,7 +8,7 @@
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
@@ -29,41 +29,38 @@ enum
 {
   CREATE  = 1 << 0,
   DESTROY = 1 << 1,
-  CHECK   = 1 << 2
+  CHECK	  = 1 << 2
 };
 
 
-static void         gtk_menu_factory_create            (GtkMenuFactory *factory,
+static void	    gtk_menu_factory_create	       (GtkMenuFactory *factory,
 							GtkMenuEntry   *entry,
 							GtkWidget      *parent,
-							const char     *path);
-static void         gtk_menu_factory_remove            (GtkMenuFactory *factory,
+							const gchar    *path);
+static void	    gtk_menu_factory_remove	       (GtkMenuFactory *factory,
 							GtkWidget      *parent,
-							const char     *path);
+							const gchar    *path);
 static GtkWidget*   gtk_menu_factory_make_widget       (GtkMenuFactory *factory);
-static GtkMenuPath* gtk_menu_factory_get               (GtkWidget      *parent,
-							const char     *path,
-							int             flags);
+static GtkMenuPath* gtk_menu_factory_get	       (GtkWidget      *parent,
+							const gchar    *path,
+							gint		flags);
 static GtkMenuPath* gtk_menu_factory_find_recurse      (GtkMenuFactory *factory,
 							GtkWidget      *parent,
-							const char     *path);
-static void         gtk_menu_factory_parse_accelerator (const char     *accelerator,
-							char     *accelerator_key,
-							guint8         *accelerator_mods);
+							const gchar    *path);
 
 
 GtkMenuFactory*
 gtk_menu_factory_new (GtkMenuFactoryType type)
 {
   GtkMenuFactory *factory;
-
+  
   factory = g_new (GtkMenuFactory, 1);
   factory->path = NULL;
   factory->type = type;
-  factory->table = NULL;
+  factory->accel_group = NULL;
   factory->widget = NULL;
   factory->subfactories = NULL;
-
+  
   return factory;
 }
 
@@ -72,25 +69,25 @@ gtk_menu_factory_destroy (GtkMenuFactory *factory)
 {
   GtkMenuFactory *subfactory;
   GList *tmp_list;
-
+  
   g_return_if_fail (factory != NULL);
-
+  
   if (factory->path)
     g_free (factory->path);
-
+  
   tmp_list = factory->subfactories;
   while (tmp_list)
     {
       subfactory = tmp_list->data;
       tmp_list = tmp_list->next;
-
+      
       gtk_menu_factory_destroy (subfactory);
     }
-
-  if (factory->table)
+  
+  if (factory->accel_group)
     {
-      gtk_accelerator_table_unref (factory->table);
-      factory->table = NULL;
+      gtk_accel_group_unref (factory->accel_group);
+      factory->accel_group = NULL;
     }
   
   if (factory->widget)
@@ -100,21 +97,21 @@ gtk_menu_factory_destroy (GtkMenuFactory *factory)
 void
 gtk_menu_factory_add_entries (GtkMenuFactory *factory,
 			      GtkMenuEntry   *entries,
-			      int             nentries)
+			      int	      nentries)
 {
   int i;
-
+  
   g_return_if_fail (factory != NULL);
   g_return_if_fail (entries != NULL);
   g_return_if_fail (nentries > 0);
-
+  
   if (!factory->widget)
     {
       factory->widget = gtk_menu_factory_make_widget (factory);
       gtk_widget_ref  (factory->widget);
       gtk_object_sink (GTK_OBJECT (factory->widget));
     }
-
+  
   for (i = 0; i < nentries; i++)
     gtk_menu_factory_create (factory, &entries[i], factory->widget, entries[i].path);
 }
@@ -122,30 +119,30 @@ gtk_menu_factory_add_entries (GtkMenuFactory *factory,
 void
 gtk_menu_factory_add_subfactory (GtkMenuFactory *factory,
 				 GtkMenuFactory *subfactory,
-				 const char     *path)
+				 const char	*path)
 {
   g_return_if_fail (factory != NULL);
   g_return_if_fail (subfactory != NULL);
   g_return_if_fail (path != NULL);
-
+  
   if (subfactory->path)
     g_free (subfactory->path);
   subfactory->path = g_strdup (path);
-
+  
   factory->subfactories = g_list_append (factory->subfactories, subfactory);
 }
 
 void
 gtk_menu_factory_remove_paths (GtkMenuFactory  *factory,
-			       char           **paths,
-			       int              npaths)
+			       char	      **paths,
+			       int		npaths)
 {
   int i;
-
+  
   g_return_if_fail (factory != NULL);
   g_return_if_fail (paths != NULL);
   g_return_if_fail (npaths > 0);
-
+  
   if (factory->widget)
     {
       for (i = 0; i < npaths; i++)
@@ -155,15 +152,15 @@ gtk_menu_factory_remove_paths (GtkMenuFactory  *factory,
 
 void
 gtk_menu_factory_remove_entries (GtkMenuFactory *factory,
-				 GtkMenuEntry   *entries,
-				 int             nentries)
+				 GtkMenuEntry	*entries,
+				 int		 nentries)
 {
   int i;
-
+  
   g_return_if_fail (factory != NULL);
   g_return_if_fail (entries != NULL);
   g_return_if_fail (nentries > 0);
-
+  
   if (factory->widget)
     {
       for (i = 0; i < nentries; i++)
@@ -174,12 +171,12 @@ gtk_menu_factory_remove_entries (GtkMenuFactory *factory,
 void
 gtk_menu_factory_remove_subfactory (GtkMenuFactory *factory,
 				    GtkMenuFactory *subfactory,
-				    const char     *path)
+				    const char	   *path)
 {
   g_return_if_fail (factory != NULL);
   g_return_if_fail (subfactory != NULL);
   g_return_if_fail (path != NULL);
-
+  
   g_warning ("FIXME: gtk_menu_factory_remove_subfactory");
 }
 
@@ -189,29 +186,29 @@ gtk_menu_factory_find (GtkMenuFactory *factory,
 {
   g_return_val_if_fail (factory != NULL, NULL);
   g_return_val_if_fail (path != NULL, NULL);
-
+  
   return gtk_menu_factory_find_recurse (factory, factory->widget, path);
 }
 
 
 static void
 gtk_menu_factory_create (GtkMenuFactory *factory,
-			 GtkMenuEntry   *entry,
-			 GtkWidget      *parent,
-			 const char     *path)
+			 GtkMenuEntry	*entry,
+			 GtkWidget	*parent,
+			 const char	*path)
 {
   GtkMenuFactory *subfactory;
   GtkMenuPath *menu_path;
   GtkWidget *menu;
   GList *tmp_list;
   char tmp_path[256];
-  char accelerator_key;
-  guint8 accelerator_mods;
-  char *p;
-
+  guint accelerator_key;
+  guint accelerator_mods;
+  gchar *p;
+  
   g_return_if_fail (factory != NULL);
   g_return_if_fail (entry != NULL);
-
+  
   /* If 'path' is empty, then simply return.
    */
   if (!path || path[0] == '\0')
@@ -223,11 +220,11 @@ gtk_menu_factory_create (GtkMenuFactory *factory,
       g_warning ("gtk_menu_factory_create(): argument `path' exceeds maximum size.");
       return;
     }
-
+  
   /* Strip off the next part of the path.
    */
   p = strchr (path, '/');
-
+  
   /* If this is the last part of the path ('p' is
    *  NULL), then we create an item.
    */
@@ -248,25 +245,26 @@ gtk_menu_factory_create (GtkMenuFactory *factory,
 	  else
 	    menu_path = gtk_menu_factory_get (parent, path, CREATE);
 	  entry->widget = menu_path->widget;
-
+	  
 	  if (strcmp (path, "<nothing>") == 0)
 	    gtk_widget_hide (entry->widget);
-
+	  
 	  if (entry->accelerator)
 	    {
-	      gtk_menu_factory_parse_accelerator (entry->accelerator,
-						  &accelerator_key,
-						  &accelerator_mods);
-	      if (!factory->table)
-		factory->table = gtk_accelerator_table_new ();
+	      gtk_accelerator_parse (entry->accelerator,
+				     &accelerator_key,
+				     &accelerator_mods);
+	      if (!factory->accel_group)
+		factory->accel_group = gtk_accel_group_new ();
 	      
-	      gtk_widget_install_accelerator (menu_path->widget,
-					      factory->table,
-					      "activate",
-					      accelerator_key,
-					      accelerator_mods);
+	      gtk_widget_add_accelerator (menu_path->widget,
+					  "activate",
+					  factory->accel_group,
+					  accelerator_key,
+					  accelerator_mods,
+					  GTK_ACCEL_VISIBLE);
 	    }
-
+	  
 	  if (entry->callback)
 	    gtk_signal_connect (GTK_OBJECT (menu_path->widget), "activate",
 				(GtkSignalFunc) entry->callback,
@@ -277,7 +275,7 @@ gtk_menu_factory_create (GtkMenuFactory *factory,
     {
       strncpy (tmp_path, path, (unsigned int) ((long) p - (long) path));
       tmp_path[(long) p - (long) path] = '\0';
-
+      
       menu_path = gtk_menu_factory_get (parent, tmp_path, 0);
       if (!menu_path)
 	{
@@ -286,7 +284,7 @@ gtk_menu_factory_create (GtkMenuFactory *factory,
 	    {
 	      subfactory = tmp_list->data;
 	      tmp_list = tmp_list->next;
-
+	      
 	      if (subfactory->path &&
 		  (strcmp (subfactory->path, tmp_path) == 0))
 		{
@@ -296,36 +294,36 @@ gtk_menu_factory_create (GtkMenuFactory *factory,
 		      gtk_widget_ref  (subfactory->widget);
 		      gtk_object_sink (GTK_OBJECT (subfactory->widget));
 		    }
-
+		  
 		  gtk_menu_factory_create (subfactory, entry, subfactory->widget, p + 1);
 		  return;
 		}
 	    }
-
+	  
 	  menu_path = gtk_menu_factory_get (parent, tmp_path, CREATE);
 	}
-
+      
       entry->widget = menu_path->widget;
       menu = GTK_MENU_ITEM (menu_path->widget)->submenu;
-
+      
       if (!menu)
 	{
 	  menu = gtk_menu_new ();
 	  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_path->widget), menu);
-
-	  if (!factory->table)
-	    factory->table = gtk_accelerator_table_new ();
-	  gtk_menu_set_accelerator_table (GTK_MENU (menu), factory->table);
+	  
+	  if (!factory->accel_group)
+	    factory->accel_group = gtk_accel_group_new ();
+	  gtk_menu_set_accel_group (GTK_MENU (menu), factory->accel_group);
 	}
-
+      
       gtk_menu_factory_create (factory, entry, menu, p + 1);
     }
 }
 
 static void
 gtk_menu_factory_remove (GtkMenuFactory *factory,
-			 GtkWidget      *parent,
-			 const char     *path)
+			 GtkWidget	*parent,
+			 const char	*path)
 {
   GtkMenuFactory *subfactory;
   GtkMenuPath *menu_path;
@@ -333,7 +331,7 @@ gtk_menu_factory_remove (GtkMenuFactory *factory,
   GList *tmp_list;
   char tmp_path[256];
   char *p;
-
+  
   if (!path || path[0] == '\0')
     return;
   else if (strlen (path) >= 250)
@@ -345,7 +343,7 @@ gtk_menu_factory_remove (GtkMenuFactory *factory,
     }
   
   p = strchr (path, '/');
-
+  
   if (!p)
     {
       if (parent)
@@ -355,7 +353,7 @@ gtk_menu_factory_remove (GtkMenuFactory *factory,
     {
       strncpy (tmp_path, path, (unsigned int) ((long) p - (long) path));
       tmp_path[(long) p - (long) path] = '\0';
-
+      
       menu_path = gtk_menu_factory_get (parent, tmp_path, 0);
       if (!menu_path)
 	{
@@ -364,7 +362,7 @@ gtk_menu_factory_remove (GtkMenuFactory *factory,
 	    {
 	      subfactory = tmp_list->data;
 	      tmp_list = tmp_list->next;
-
+	      
 	      if (subfactory->path &&
 		  (strcmp (subfactory->path, tmp_path) == 0))
 		{
@@ -387,17 +385,17 @@ static GtkWidget*
 gtk_menu_factory_make_widget (GtkMenuFactory *factory)
 {
   GtkWidget *widget;
-
+  
   g_return_val_if_fail (factory != NULL, NULL);
-
+  
   switch (factory->type)
     {
     case GTK_MENU_FACTORY_MENU:
       widget = gtk_menu_new ();
-
-      if (!factory->table)
-	factory->table = gtk_accelerator_table_new ();
-      gtk_menu_set_accelerator_table (GTK_MENU (widget), factory->table);
+      
+      if (!factory->accel_group)
+	factory->accel_group = gtk_accel_group_new ();
+      gtk_menu_set_accel_group (GTK_MENU (widget), factory->accel_group);
       return widget;
     case GTK_MENU_FACTORY_MENU_BAR:
       return gtk_menu_bar_new ();
@@ -405,24 +403,24 @@ gtk_menu_factory_make_widget (GtkMenuFactory *factory)
       g_error ("not implemented");
       break;
     }
-
+  
   return NULL;
 }
 
 static GtkMenuPath*
 gtk_menu_factory_get (GtkWidget *parent,
 		      const char *path,
-		      int        flags)
+		      int	 flags)
 {
   GtkMenuPath *menu_path;
   GList *tmp_list;
-
+  
   tmp_list = gtk_object_get_user_data (GTK_OBJECT (parent));
   while (tmp_list)
     {
       menu_path = tmp_list->data;
       tmp_list = tmp_list->next;
-
+      
       if (strcmp (menu_path->path, path) == 0)
 	{
 	  if (flags & DESTROY)
@@ -430,11 +428,11 @@ gtk_menu_factory_get (GtkWidget *parent,
 	      tmp_list = gtk_object_get_user_data (GTK_OBJECT (parent));
 	      tmp_list = g_list_remove (tmp_list, menu_path);
 	      gtk_object_set_user_data (GTK_OBJECT (parent), tmp_list);
-
+	      
 	      gtk_widget_destroy (menu_path->widget);
 	      g_free (menu_path->path);
 	      g_free (menu_path);
-
+	      
 	      return NULL;
 	    }
 	  else
@@ -443,28 +441,28 @@ gtk_menu_factory_get (GtkWidget *parent,
 	    }
 	}
     }
-
+  
   if (flags & CREATE)
     {
       menu_path = g_new (GtkMenuPath, 1);
       menu_path->path = g_strdup (path);
-
+      
       if (flags & CHECK)
 	menu_path->widget = gtk_check_menu_item_new_with_label (path);
       else
 	menu_path->widget = gtk_menu_item_new_with_label (path);
-
+      
       gtk_container_add (GTK_CONTAINER (parent), menu_path->widget);
       gtk_object_set_user_data (GTK_OBJECT (menu_path->widget), NULL);
       gtk_widget_show (menu_path->widget);
-
+      
       tmp_list = gtk_object_get_user_data (GTK_OBJECT (parent));
       tmp_list = g_list_prepend (tmp_list, menu_path);
       gtk_object_set_user_data (GTK_OBJECT (parent), tmp_list);
-
+      
       return menu_path;
     }
-
+  
   return NULL;
 }
 
@@ -479,7 +477,7 @@ gtk_menu_factory_find_recurse (GtkMenuFactory *factory,
   GList *tmp_list;
   char tmp_path[256];
   char *p;
-
+  
   if (!path || path[0] == '\0')
     return NULL;
   else if (strlen (path) >= 250)
@@ -491,7 +489,7 @@ gtk_menu_factory_find_recurse (GtkMenuFactory *factory,
     }
   
   p = strchr (path, '/');
-
+  
   if (!p)
     {
       if (parent)
@@ -501,7 +499,7 @@ gtk_menu_factory_find_recurse (GtkMenuFactory *factory,
     {
       strncpy (tmp_path, path, (unsigned int) ((long) p - (long) path));
       tmp_path[(long) p - (long) path] = '\0';
-
+      
       menu_path = gtk_menu_factory_get (parent, tmp_path, 0);
       if (!menu_path)
 	{
@@ -510,7 +508,7 @@ gtk_menu_factory_find_recurse (GtkMenuFactory *factory,
 	    {
 	      subfactory = tmp_list->data;
 	      tmp_list = tmp_list->next;
-
+	      
 	      if (subfactory->path &&
 		  (strcmp (subfactory->path, tmp_path) == 0))
 		{
@@ -519,54 +517,14 @@ gtk_menu_factory_find_recurse (GtkMenuFactory *factory,
 		  return gtk_menu_factory_find_recurse (subfactory, subfactory->widget, p + 1);
 		}
 	    }
-
+	  
 	  return NULL;
 	}
-
+      
       menu = GTK_MENU_ITEM (menu_path->widget)->submenu;
       if (menu)
 	return gtk_menu_factory_find_recurse (factory, menu, p + 1);
     }
-
+  
   return NULL;
 }
-
-static void
-gtk_menu_factory_parse_accelerator (const char   *accelerator,
-                                    char   *accelerator_key,
-				    guint8 *accelerator_mods)
-{
-  int done;
-
-  g_return_if_fail (accelerator != NULL);
-  g_return_if_fail (accelerator_key != NULL);
-  g_return_if_fail (accelerator_mods != NULL);
-
-  *accelerator_key = 0;
-  *accelerator_mods = 0;
-
-  done = FALSE;
-  while (!done)
-    {
-      if (strncmp (accelerator, "<shift>", 7) == 0)
-        {
-          accelerator += 7;
-          *accelerator_mods |= GDK_SHIFT_MASK;
-        }
-      else if (strncmp (accelerator, "<alt>", 5) == 0)
-        {
-          accelerator += 5;
-          *accelerator_mods |= GDK_MOD1_MASK;
-        }
-      else if (strncmp (accelerator, "<control>", 9) == 0)
-        {
-          accelerator += 9;
-          *accelerator_mods |= GDK_CONTROL_MASK;
-        }
-      else
-        {
-          done = TRUE;
-          *accelerator_key = accelerator[0];
-        }
-    }
-}
